#include <unistd.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/sysinfo.h>
#include <fcntl.h>
#include <signal.h>
#include <time.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <string>
#include <memory>
#include <iostream>
#include <thread>
#include <list>
#include <mutex>
#include <vector>
#include <chrono>
#include <algorithm>
#include <opencv2/core.hpp>
#include <opencv2/imgproc.hpp>
#include <opencv2/imgcodecs.hpp>
#include <opencv2/highgui.hpp>
#include <iostream>
#include "tof_sdk_typedef.h"
#include "tof_dev_sdk.h"

// ----------------
// Utility function
// ----------------

struct StreamCallBackParam{
  TofDeviceInfo struCaps;

  UINT32 tof_frame_count;
  UINT32 rgb_frame_count;
  
  cv::Mat rgb = cv::Mat(1080, 1920, CV_8UC3);
  cv::Mat rgbd = cv::Mat(224,109 , CV_8UC3);
  cv::Mat depth = cv::Mat(224, 109, CV_32FC3);
  cv::Mat gray = cv::Mat(224, 109, CV_8UC1);


};


static void fnTofStream(TofFrameData *tofFrameData, void *pUserData)
{
  StreamCallBackParam *pStreamParam = (StreamCallBackParam *)pUserData;
  (pStreamParam->tof_frame_count)++;
  //
  const UINT32 width = tofFrameData->frameWidth;
  const UINT32 height = tofFrameData->frameHeight;

  cv::Mat gray(height, width, CV_8UC1, tofFrameData->pGrayData);
  cv::Mat rgbd(height, width, CV_8UC3, tofFrameData->pRgbD);
  cv::Mat depth(height, width, CV_32FC3, tofFrameData->pPointData);
  pStreamParam->gray = gray;
  pStreamParam->rgbd = rgbd;

  // convert depth from 32FC3 to 8UC1, with min max normalization
  // cv::Mat depth(height, width, CV_32FC3, tofFrameData->pPointData);
  // cv::Mat depth_8UC1;
  // min max normalization
  cv::normalize(depth, depth, 0, 255, cv::NORM_MINMAX, CV_8UC3);
  cv::cvtColor(depth, depth, cv::COLOR_BGR2GRAY);
  // pStreamParam->depth = depth_8UC1;
  pStreamParam->depth = depth;
  
  // cv::Mat depth_8UC1;
  // // min max normalization
  // cv::normalize(depth, depth, 0, 255, cv::NORM_MINMAX, CV_8UC1);
  
  return;
}

static void fnRgbStream(RgbFrameData *rgbFrameData, void *pUserData)
{
  StreamCallBackParam *pStreamParam = (StreamCallBackParam *)pUserData;
  (pStreamParam->rgb_frame_count)++;	
  //
  const UINT32 width = rgbFrameData->frameWidth;
  const UINT32 height = rgbFrameData->frameHeight;

  cv::Mat rgb(height, width, CV_8UC3, rgbFrameData->pFrameData);
  cv::Mat bgr(height, width, CV_8UC3);
  cv::cvtColor(rgb, bgr, cv::COLOR_RGB2BGR);

  pStreamParam->rgb = bgr;
  return;
}

static void PrintDevInfo(TofDeviceInfo *pTofDeviceInfo)
{
  printf("Dev Info:==================================\n");
  printf(">>  szDevName=%s.\n", pTofDeviceInfo->szDevName);
  printf(">>  szDevId=%s.\n", pTofDeviceInfo->szDevId);
  printf(">>  szFirmwareVersion=%s.\n", pTofDeviceInfo->szFirmwareVersion);
  printf("Dev Info==================================\n\n");
}
static void CallBackTofDeviceStatus(TOFDEV_STATUS tofDevStatus, void *pUserData)
{
	printf("device status: 0x08%x.\n", tofDevStatus);
  pUserData = pUserData;

	if (TOFDEV_STATUS_DEV_BROKEN == tofDevStatus)
	{
		printf("a device connection is broken!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
	}

}
static TofDeviceCapability *GetTofModeCaps(TofDeviceInfo &struCaps, const TOF_MODE tofMode)
{
  for (UINT32 i = 0; ((i < struCaps.capCnt) && (i < TOF_MAX_CAPS_CNT)); i++)
  {
    if (tofMode == struCaps.cap[i].tofMode)
    {
      return (&(struCaps.cap[i]));
    }
  }
  return NULL;
}