import os
from ament_index_python.packages import get_package_share_directory
from launch import LaunchDescription
from launch_ros.actions import Node, LifecycleNode
from launch.actions import EmitEvent, DeclareLaunchArgument
from launch_ros.events.lifecycle import ChangeState
from launch.events import matches_action
from lifecycle_msgs.msg import Transition
from launch.substitutions import LaunchConfiguration, PathJoinSubstitution, TextSubstitution


def generate_launch_description():
    config_file = LaunchConfiguration("config_file")
    config_file_launch_arg = DeclareLaunchArgument(
        "config_file",
        default_value="pc.yaml",
        description="Config file to load",
    )
    node_name = LaunchConfiguration("node_name")
    node_name_launch_arg = DeclareLaunchArgument(
        "node_name",
        default_value="camera_node",
        description="Config file to load",
    )
    camera = LifecycleNode(
        package="camera_cpp",
        executable="camera_cpp",
        name=node_name,
        parameters=[
            PathJoinSubstitution([
                get_package_share_directory("camera_cpp"), 
                "config", 
                config_file
        ])]
    )
    return LaunchDescription([
        config_file_launch_arg,
        node_name_launch_arg,
        camera,
        EmitEvent(event=ChangeState(
            lifecycle_node_matcher=matches_action(camera),
            transition_id=Transition.TRANSITION_CONFIGURE,
        )),
        EmitEvent(event=ChangeState(
            lifecycle_node_matcher=matches_action(camera),
            transition_id=Transition.TRANSITION_ACTIVATE,
        ))
    ])
